import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

import org.hornetq.api.core.HornetQException;
import org.hornetq.api.core.TransportConfiguration;
import org.hornetq.api.core.client.ClientConsumer;
import org.hornetq.api.core.client.ClientMessage;
import org.hornetq.api.core.client.ClientProducer;
import org.hornetq.api.core.client.ClientSession;
import org.hornetq.api.core.client.ClientSessionFactory;
import org.hornetq.api.core.client.HornetQClient;
import org.hornetq.api.core.client.ServerLocator;
import org.hornetq.core.remoting.impl.netty.NettyConnectorFactory;

public class TestHornetQ {

	private static final int totalMessages = 1000000;
	static AtomicInteger messageId = new AtomicInteger(0);
	private static AtomicInteger currentRetry = new AtomicInteger(0);
	static AtomicInteger consumedCounter = new AtomicInteger(0);
	private static int retryLimit = 10;
	
//	public static void main(String[] args) {
	public static void mainProduce(String[] args) {
		String host = "192.168.1.95";
		int port = 5445;
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(org.hornetq.core.remoting.impl.netty.TransportConstants.HOST_PROP_NAME, host);
		params.put(org.hornetq.core.remoting.impl.netty.TransportConstants.PORT_PROP_NAME, port);
		TransportConfiguration server = new TransportConfiguration(NettyConnectorFactory.class.getName(), params);
		ServerLocator locator = HornetQClient.createServerLocatorWithHA(server);
		String queueName = "jms.queue.QUEUE_EMAIL_OUT";
		
		String subject = "Email testing for HornetQ";
		int randomPriority = 1;
		
		ClientSessionFactory factory = null;
		ClientSession session = null;
		try {
			factory = locator.createSessionFactory();
			session = factory.createSession(true, true); // Set autocommit and autoacknowledge both to true
			ClientProducer producer = session.createProducer(queueName);
			for(int i=0;i<totalMessages;i++) {
				String str = subject+" ["+messageId.incrementAndGet()+"]";
				ClientMessage message = session.createMessage(true);
				message.putStringProperty("msg", str);
				message.setPriority((byte)randomPriority);
				message.setExpiration(0);
				producer.send(message);
				System.out.println("PRODUCED: "+str);
			}
			if(randomPriority >= 9) {
				randomPriority = 1;
			} else {
				randomPriority += 2;
			}
		} catch (Exception e) {
			System.out.println("Unexpected exception!!!");
			e.printStackTrace();
		} finally {
			if(session != null) {
				try {
					session.close();
				} catch (HornetQException e) {
					System.out.println("Unable to close client session!!!");
					e.printStackTrace();
				}
			}
			if(factory != null) {
				factory.close();
			}
		}
	}
	
	public static void main(String[] args) {
//	public static void mainConsume(String[] args) {
		String host = "192.168.1.99";
		int port = 5445;
		Map<String, Object> params = new HashMap<String, Object>();
		params.put(org.hornetq.core.remoting.impl.netty.TransportConstants.HOST_PROP_NAME, host);
		params.put(org.hornetq.core.remoting.impl.netty.TransportConstants.PORT_PROP_NAME, port);
		TransportConfiguration server = new TransportConfiguration(NettyConnectorFactory.class.getName(), params);
		ServerLocator locator = HornetQClient.createServerLocatorWithHA(server);
		String queueName = "jms.queue.QUEUE_EMAIL_OUT";
		
		ClientSessionFactory factory = null;
		ClientSession session = null;
		
		try {
			factory = locator.createSessionFactory();
			session = factory.createSession(true, true); // Set autocommit and autoacknowledge both to true
			session.start();
			ClientConsumer consumer = session.createConsumer(queueName);
			while(true) {
				ClientMessage message = consumer.receive(1000);
				if(message != null) {
					System.out.println(String.format("Message '%s' Counter [%s]", message.getStringProperty("msg"), consumedCounter.incrementAndGet()));
					currentRetry.set(0); // Reset retry counter
				} else {
					currentRetry.incrementAndGet();
					if(currentRetry.get() >= retryLimit) {
						System.out.println("No message, stop consume..");
						break;
					}
				}
			}
		} catch (Exception e) {
			System.out.println("Unexpected exception!!!");
			e.printStackTrace();
		} finally {
			if(session != null) {
				try {
					session.close();
				} catch (HornetQException e) {
					System.out.println("Unable to close client session!!!");
					e.printStackTrace();
				}
			}
			if(factory != null) {
				factory.close();
			}
		}
	}
	
}
